public static class Acer
{
	#region Notebook Acer Columbia : class Columbia

	public static class Columbia
	{
		public static class BRIGHTNESS
		{
			public const string description = "LCD Brightness Control";
			public const bool default_enable = true;
			public const int icon = 1;

			public const int minimum_brightness_level = 10;
			public const int maximum_brightness_level = 100;
			public const int brightness_level_step = 10;

			public static class BRIGHTNESS_LEVEL
			{
				public const string description = "Current LCD brightness level";
				public const bool nhc_visible = true;
				public static int brightness_level = 0;
				public const string unit = "%";
			}

			public static bool set_brightness(int new_level)
			{
				return ACPI.METHOD.SetValue("_SB.PCI0.PEGP.VGA.LCD._BCM", new_level);
			}

			public static bool get_brightness(ref int level)
			{
				bool result = ACPI.METHOD.GetValue("_SB.PCI0.PEGP.VGA.LCD._BQC", ref level);     // get the brightness level
				BRIGHTNESS_LEVEL.brightness_level = level;
				return result;
			}
		}


		public static class FAN
		{
			public const string description = "ACPI fan control";
			public const bool default_enable = false;
			public const int icon = 2;
			
			public static bool settingsChanged = false;
			public static bool interpolate = false;
			public static int[] activeTemps = new int[7];
			public static int[] activeSpeeds = new int[activeTemps.Length];
			

			public static class FAN_RPM
			{
				public const string description = "Speed";
				public const bool nhc_visible = true;
				public static int fan_rpm = 0;
				public const string unit = " rpm";
				
				public static bool update()
				{
					int temp = 0;
					ACPI.FIELD.Read("_SB.AMW0.FAN1", ref temp);
					fan_rpm = temp==0xff ? 0 : 0x78000 / (temp>0 ? temp : 1);
					return true;
				}
			}


			public static class FAN_Temperature
			{
				public const string description = "Temperature";
				public const bool nhc_visible = true;
				public static int fan_temperature = 0;
				public const string unit = " C";
				
				public static bool update()
				{
					int temp0 = 0;
					int temp1 = 0;
					ACPI.METHOD.GetValue("_TZ.TZS0._TMP", ref temp0);
					ACPI.METHOD.GetValue("_TZ.TZS1._TMP", ref temp1);
					if ( temp1>temp0 ) temp0 = temp1;
					fan_temperature = (temp0 - 0xaac) / 10;
					return true;
				}
			}


			public static class FAN_EC_CONTROL
			{
				public const string description = "EC controlling";
				public const bool nhc_visible = true;
				public static int fan_ec_control = 0;
				public const string unit = "OFF|ON";
			}


			public static class ADVANCED_FAN_CONTROL
			{
				public const string description = "Advanced fan control";
				public const bool nhc_visible = true;
				public static int advanced_fan_control = 0; // if 1, then manual control is enabled
				public const string unit = "OFF|ON";

				public static bool isManual()
				{
					int isManual = 0;
					ACPI.FIELD.ReadByOffset("_SB.PCI0.LPCB.EC0.RAM", ref isManual, 0x93, 8);
					bool manual = (isManual & 0x10)!=0;  // if bit is set, manual fan control is enabled
					FAN_EC_CONTROL.fan_ec_control = manual ? 0 : 1;
					return manual;
				}

				public static bool setManual(bool manual)
				{
					int temp = 0;
					FAN_EC_CONTROL.fan_ec_control = manual ? 0 : 1;
					ACPI.FIELD.ReadByOffset("_SB.PCI0.LPCB.EC0.RAM", ref temp, 0x93, 8);
					bool current = (temp & 0x10)!=0;
					if ( current!=manual ) {
						temp = manual ? temp | 0x10 : temp & (~0x10);
						return ACPI.FIELD.WriteByOffset("_SB.PCI0.LPCB.EC0.RAM", temp, 0x93, 8);
					}
					return true;
				}

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 1;
					public const int nhc_default_value = 0;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						if ( initialisation ) return true;
						return advanced_fan_control==0 ? ADVANCED_FAN_CONTROL.setManual(false) : true;
					}
				}
			}


			public static class FAN_ACTIVE0_TEMP
			{
				public const string description = "Active[0] trip point";
				public const bool nhc_visible = true;
				public static int fan_active0_temp = 0;
				public const string unit = " C";

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 90;
					public const int nhc_default_value = 32;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeTemps[0] = FAN_ACTIVE0_TEMP.fan_active0_temp;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE1_TEMP
			{
				public const string description = "Active[1] trip point";
				public const bool nhc_visible = true;
				public static int fan_active1_temp = 0;
				public const string unit = " C";

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 90;
					public const int nhc_default_value = 40;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeTemps[1] = FAN_ACTIVE1_TEMP.fan_active1_temp;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE2_TEMP
			{
				public const string description = "Active[2] trip point";
				public const bool nhc_visible = true;
				public static int fan_active2_temp = 0;
				public const string unit = " C";

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 90;
					public const int nhc_default_value = 46;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeTemps[2] = FAN_ACTIVE2_TEMP.fan_active2_temp;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE3_TEMP
			{
				public const string description = "Active[3] trip point";
				public const bool nhc_visible = true;
				public static int fan_active3_temp = 0;
				public const string unit = " C";

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 90;
					public const int nhc_default_value = 47;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeTemps[3] = FAN_ACTIVE3_TEMP.fan_active3_temp;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE4_TEMP
			{
				public const string description = "Active[4] trip point";
				public const bool nhc_visible = true;
				public static int fan_active4_temp = 0;
				public const string unit = " C";

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 90;
					public const int nhc_default_value = 50;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeTemps[4] = FAN_ACTIVE4_TEMP.fan_active4_temp;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE5_TEMP
			{
				public const string description = "Active[5] trip point";
				public const bool nhc_visible = true;
				public static int fan_active5_temp = 0;
				public const string unit = " C";

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 90;
					public const int nhc_default_value = 55;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeTemps[5] = FAN_ACTIVE5_TEMP.fan_active5_temp;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE6_TEMP
			{
				public const string description = "Active[6] trip point";
				public const bool nhc_visible = true;
				public static int fan_active6_temp = 0;
				public const string unit = " C";

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 90;
					public const int nhc_default_value = 60;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeTemps[6] = FAN_ACTIVE6_TEMP.fan_active6_temp;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE0_SPEED
			{
				public const string description = "Active[0] fan speed";
				public const bool nhc_visible = true;
				public static int fan_active0_speed = 0;

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 255;
					public const int nhc_default_value = 0;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeSpeeds[0] = FAN_ACTIVE0_SPEED.fan_active0_speed;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE1_SPEED
			{
				public const string description = "Active[1] fan speed";
				public const bool nhc_visible = true;
				public static int fan_active1_speed = 0;

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 255;
					public const int nhc_default_value = 1;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeSpeeds[1] = FAN_ACTIVE1_SPEED.fan_active1_speed;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE2_SPEED
			{
				public const string description = "Active[2] fan speed";
				public const bool nhc_visible = true;
				public static int fan_active2_speed = 0;

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 255;
					public const int nhc_default_value = 1;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeSpeeds[2] = FAN_ACTIVE2_SPEED.fan_active2_speed;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE3_SPEED
			{
				public const string description = "Active[3] fan speed";
				public const bool nhc_visible = true;
				public static int fan_active3_speed = 0;

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 255;
					public const int nhc_default_value = 20;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeSpeeds[3] = FAN_ACTIVE3_SPEED.fan_active3_speed;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE4_SPEED
			{
				public const string description = "Active[4] fan speed";
				public const bool nhc_visible = true;
				public static int fan_active4_speed = 0;

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 255;
					public const int nhc_default_value = 40;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeSpeeds[4] = FAN_ACTIVE4_SPEED.fan_active4_speed;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE5_SPEED
			{
				public const string description = "Active[5] fan speed";
				public const bool nhc_visible = true;
				public static int fan_active5_speed = 0;

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 255;
					public const int nhc_default_value = 60;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeSpeeds[5] = FAN_ACTIVE5_SPEED.fan_active5_speed;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_ACTIVE6_SPEED
			{
				public const string description = "Active[6] fan speed";
				public const bool nhc_visible = true;
				public static int fan_active6_speed = 0;

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 255;
					public const int nhc_default_value = 0xff - 0xaf;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.activeSpeeds[6] = FAN_ACTIVE6_SPEED.fan_active6_speed;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_INTERPOLATE
			{
				public const string description = "Interpolate";
				public const bool nhc_visible = true;
				public static int fan_interpolate = 0;
				public const string unit = "OFF|ON";

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 1;
					public const int nhc_default_value = 0;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						FAN.interpolate = FAN_INTERPOLATE.fan_interpolate!=0;
						FAN.settingsChanged = true;
						return true;
					}
				}
			}


			public static class FAN_SPEED
			{
				public const string description = "Desired fan speed (0 to 255)";
				public const bool nhc_visible = true;
				public static int fan_speed = 0;
				public const string unit = "";
				public const int limit = 145;

				public static bool update()
				{
					return ACPI.FIELD.WriteByOffset("_SB.PCI0.LPCB.EC0.RAM", 0xff - fan_speed, 0x94, 8);
				}

				public static class NHC_WRITE
				{
					public const bool nhc_write = true;
					public const int nhc_minimum_value = 0;
					public const int nhc_maximum_value = 255;
					public const int nhc_default_value = 0; //0xff - 0xaf;
					public static bool initialisation = false;

					public static bool nhc_on_write()
					{
						if ( initialisation ) return true;
						return ADVANCED_FAN_CONTROL.advanced_fan_control!=0 ? FAN_SPEED.update() : true;
					}
				}
			}


			public static bool init()
			{
				FAN_Temperature.update();
				FAN_EC_CONTROL.fan_ec_control = ADVANCED_FAN_CONTROL.isManual() ? 0 : 1;
				return true;
			}

			
			// make sure the inbuilt fan control is enabled when NHC or the ACPI control system shut down
			public static bool close()
			{
				return ADVANCED_FAN_CONTROL.setManual(false);
			}


			public static bool refresh()
			{
				FAN_RPM.update();
				
				// make sure (really, really) we never leave the fan disabled if we don't use it
				if ( ADVANCED_FAN_CONTROL.advanced_fan_control==0 ) {
					ADVANCED_FAN_CONTROL.setManual(false);
					return true;
				}

				// find active zone
				int oldTemp  = FAN_Temperature.fan_temperature;
				FAN_Temperature.update();
				int temp = FAN_Temperature.fan_temperature;
				int zone = 0;
				for ( ; zone<activeTemps.Length && activeTemps[zone]!=temp; zone++ ) {
					if ( temp<activeTemps[zone] ) {
						zone--;
						break;
					}
				}
				// if temperature is below active 0 or higher than highest zone -> switch to EC later
				bool outside = zone<0 || zone>=activeTemps.Length;


				// adjust speed				
				int speed = FAN_SPEED.fan_speed;
				if ( outside ) {
					speed = activeSpeeds[zone<0 ? 0 : activeTemps.Length - 1];
				} else {
					int oldSpeed = speed;
					speed = activeSpeeds[zone];
					if ( interpolate ) {
						int temp0 = activeTemps[zone];
						if ( zone<activeTemps.Length-1 && temp>temp0 ) {
							int d = activeTemps[zone + 1] - temp0;
							if ( d>0 ) speed += (activeSpeeds[zone + 1] - speed) * (temp - temp0) / d;
						}
					} else {
						if ( oldSpeed>speed ) {
							if ( zone<activeTemps.Length - 1 ) speed = activeSpeeds[zone + 1];
						}
					}
				}
				
				// TODO: if it is outside, we could also use the speed given by the EC instead of the set value
				FAN_SPEED.fan_speed = speed;
				
				// if temperature is below active 0 or higher than highest zone -> switch to EC
				if ( outside ) {
					ADVANCED_FAN_CONTROL.setManual(false);
					return true;
				}
				
				// make sure manual setting is enabled
				ADVANCED_FAN_CONTROL.setManual(true);
				FAN_SPEED.update();
				return true;
			}

		}


		public static class TZ02
		{
			public const string description = "Mainboard temperature 1";
			public const bool default_enable = true;
			public const int icon = 5;
			public const bool thermal_zone = true;

			public static class _TMP
			{
				public const string description = "Current temperature";
				public const bool nhc_visible = true;
				public static int _tmp = 0;
				public const string unit = "C";
			}

			public static class _CRT
			{
				public const string description = "Critical temperature";
				public const bool nhc_visible = true;
				public static int _crt = 0;
				public const string unit = "C";
			}

			public static class _PSV
			{
				public const string description = "Passive cooling temperature";
				public const bool nhc_visible = true;
				public static int _psv = 0;
				public const string unit = "C";
			}

			public static bool init()
			{
				ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.THS2", ref _TMP._tmp);
				ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.TSC2", ref _CRT._crt);
				return ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.TSP2", ref _PSV._psv);
			}

			public const int refresh_interval = 5;
			public static bool refresh()
			{
				return ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.THS2", ref _TMP._tmp);
			}
		}


		public static class TZ03
		{
			public const string description = "Mainboard temperature 2";
			public const bool default_enable = true;
			public const int icon = 5;
			public const bool thermal_zone = true;

			public static class _TMP
			{
				public const string description = "Current temperature";
				public const bool nhc_visible = true;
				public static int _tmp = 0;
				public const string unit = "C";
			}

			public static class _CRT
			{
				public const string description = "Critical temperature";
				public const bool nhc_visible = true;
				public static int _crt = 0;
				public const string unit = "C";
			}

			public static class _PSV
			{
				public const string description = "Passive cooling temperature";
				public const bool nhc_visible = true;
				public static int _psv = 0;
				public const string unit = "C";
			}

			public static bool init()
			{
				ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.THS3", ref _TMP._tmp);
				ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.TSC3", ref _CRT._crt);
				return ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.TSP3", ref _PSV._psv);
			}

			public const int refresh_interval = 5;
			public static bool refresh()
			{
				return ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.THS3", ref _TMP._tmp);
			}
		}


		public static class TZ04
		{
			public const string description = "Thermal zone 5";
			public const bool default_enable = true;
			public const int icon = 5;
			public const bool thermal_zone = true;

			public static class _TMP
			{
				public const string description = "Current temperature";
				public const bool nhc_visible = true;
				public static int _tmp = 0;
				public const string unit = "C";
			}

			public static class _CRT
			{
				public const string description = "Critical temperature";
				public const bool nhc_visible = true;
				public static int _crt = 0;
				public const string unit = "C";
			}

			public static class _PSV
			{
				public const string description = "Passive cooling temperature";
				public const bool nhc_visible = true;
				public static int _psv = 0;
				public const string unit = "C";
			}

			public static bool init()
			{
				ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.THS4", ref _TMP._tmp);
				ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.TSC4", ref _CRT._crt);
				return ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.TSP4", ref _PSV._psv);
			}

			public const int refresh_interval = 5;
			public static bool refresh()
			{
				return ACPI.FIELD.Read("_SB.PCI0.LPCB.EC0.THS4", ref _TMP._tmp);
			}
		}
	}

	#endregion
}